/*
	ASCII MSX2 with DEMPA Sensor Kid Emulator 'yayaMSX2SK'

	Author : umaiboux
	Date   : 2014.12.xx-

	[ Sensor Kid ]
*/

#include "sensorkid.h"
#include "../../fileio.h"

#define MB4052_CLK(X) ((X) & 1)
#define MB4052_CS(X) ((~(X)) & 0x10)
#define MB4052_CHI(X) ((X) & 0x0C)

SENSORKID::SENSORKID(VM* parent_vm, EMU* parent_emu) : DEVICE(parent_vm, parent_emu) {
	ls373_q = (~0);	// is this OK???
	mb4052_ana = 0;
	mb4052_count = 0;
	analog[0] = 11;
	analog[1] = 22;
	analog[2] = 33;
	analog[3] = 0xFF;
}

void SENSORKID::write_io8(uint32 addr, uint32 data)
{
	if (0 == (addr & 1)) {
		switch ((ls373_q ^ data) & 0xC0) {
		case 0x00:
			break;
		default:
			put_port(data, ls373_q);
			break;
		}
		if ((MB4052_CS(~data)) & MB4052_CS(ls373_q)) {
			// start of A/D convert
			mb4052_ana = get_analog(MB4052_CHI(data));
			mb4052_count = 12;
		}
		else if ((MB4052_CS(data)) & MB4052_CS(~ls373_q)) {
			// end of A/D convert
			mb4052_count = 0;
		}
		if ((MB4052_CLK(data)) & MB4052_CLK(~ls373_q)) {
			// clock
			if (mb4052_count>0) mb4052_count--;
		}
		ls373_q = data & 0xFF;
	}
}

uint32 SENSORKID::read_io8(uint32 addr)
{
	if (0 == (addr & 1)) {
		if (mb4052_count==10) addr = 0;
		else if ((mb4052_count<10) && (mb4052_count>1)) addr = 1 & (mb4052_ana>>(mb4052_count-2));
		else addr = 1;
		return addr | 0xFE;
	}
	return 0xFC | ((ls373_q>>5)&0x02) | ((ls373_q>>7)&0x01);
}

uint8 SENSORKID::get_analog(uint8 chi)
{
	cur_time_t t;
	emu->get_host_time(&t);
	if ((t.second != t0.second)||(t.minute != t0.minute)||(t.hour != t0.hour)||(t.day != t0.day)||(t.day != t0.month)||(t.day != t0.year)) {
		FILEIO* fio = new FILEIO();
		if(fio->Fopen(emu->bios_path(_T("SENSOR.BIN")), FILEIO_READ_BINARY)) {
			fio->Fread(analog, 3, 1);
			fio->Fclose();
		}
		delete fio;
		t0 = t;
	}
	/*
	switch(chi) {
	case    0:	// ANA0 (hikari)
	case 0x08:	// ANA1 (ondo)
	case 0x04:	// ANA2 (oto)
	case 0x0C:	// ANA3 (not connected)
	}
	*/
	return analog[((chi>>1)&2) | ((chi>>3)&1)];
}

void SENSORKID::put_port(uint8 data, uint8 old)
{
	switch ((old ^ data) & 0xC0) {
	case 0xC0:
		emu->out_message(_T("Port0,1:%d,%d"), ((~data)&0x80)>>7, ((~data)&0x40)>>6);
		break;
	case 0x80:
		emu->out_message(_T("Port0:%d"), ((~data)&0x80)>>7);
		break;
	case 0x40:
		emu->out_message(_T("Port1:%d"), ((~data)&0x40)>>6);
		break;
	default:
		break;
	}
}
